from ._typedattr import definenumpy, attrs_from_dict
from .const import Const
from .typext import get_attr_names


__all__ = ["definenumpy", "attrs_from_dict", "get_attr_names", "Const"]
__version__ = "0.2.6"



import os
from contextlib import contextmanager
from pathlib import Path


@contextmanager
def set_working_directory(path: Path):
    """Change directory temporarily within in the context manager.
    """
    origin = Path(os.getcwd()).absolute()
    os.chdir(path)
    yield
    os.chdir(origin)



"""Helper to run system commands and process their output."""
import subprocess
from typing import Tuple


def systemcall(call: str, verbose: bool = False) -> Tuple[str, str, int]:
    """Run a command with subprocess.Popen and process the output. This call
    is synchronous so output will only returned once the command is done.

    Args:
        call: the command to run
        verbose: verbosity (0=quiet)

    Returns:
        stdout, stderr, returncode
    """
    pipe = subprocess.PIPE
    with subprocess.Popen(call, stdout=pipe, stderr=pipe, shell=True) as process:
        out, err = process.communicate()
        retcode = process.poll()
    charset = 'utf-8'
    out = out.decode(charset)
    err = err.decode(charset)
    if verbose:
        print(f"out {out} err {err} ret {retcode}")
    return out, err, retcode


def assert_command_worked(
        errmsg: str, cmd: str, out: str, err: str, retcode: int) -> None:
    """Process the output of a systemcall and assert that the command worked.

    Args:
        errmsg: additional error info to display
        cmd: original command that was run (for display purposes)
        out: stdout of the command
        err: stderr of the command
        retcode: return code of the command

    Returns:
        None

    """
    assert retcode == 0, (
        f"command failed: {cmd}\nout: {out}\nerr: {err}\n"
        f"retcode: {retcode}\n"
        f"additional error info: {errmsg}")


def systemcall_with_assert(call: str, errmsg: str = "none", verbose: bool = False
                           ) -> Tuple[str, str, int]:
    """Run a command and assert it worked

    Args:
        call: command to run
        errmsg: additional error info to display when the command fails
        verbose: verbosity of the command

    Returns:
        stdout, stderr, returncode
    """
    out, err, retcode = systemcall(call, verbose=verbose)
    assert_command_worked(errmsg, call, out, err, retcode)
    return out, err, retcode



import datetime
from typing import Optional


def get_timestamp_for_filename(dtime: Optional[datetime.datetime] = None):
    """
    Convert datetime to timestamp for filenames.

    Args:
        dtime: Optional datetime object, will use now() if not given.

    Returns:

    """
    if dtime is None:
        dtime = datetime.datetime.now()
    ts = str(dtime).split(".", maxsplit=1)[0].replace(" ", "_")
    ts = ts.replace(":", "_").replace("-", "_")
    return ts
